﻿// ***********************************************************************
// Solution         : KC.Foundation
// Project          : KC.Foundation
// File             : RequestContext.cs
// Updated          : 2017-10-08 5:03 PM
// ***********************************************************************
// <copyright>
//     Copyright © 2016 - 2018 Kolibre Credit Team. All rights reserved.
// </copyright>
// ***********************************************************************

using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using KC.Foundation.Sys;
using KC.Foundation.Sys.Collections.Generic;
using KC.Foundation.Utilities;
using Microsoft.Extensions.DependencyInjection;
using Newtonsoft.Json;

namespace KC.Foundation
{
    /// <summary>
    ///     A request is in essence a unit of top level named server side work that can be timed and independently succeed/fail.
    /// </summary>
    public class RequestContext
    {
        private const int SuccessCode = 200;
        private const int FailureCode = 500;
        private int? _responseCode;

        private IServiceProvider _serviceProvider;

        public RequestContext()
        {
            Id = ID.NewSequentialGuid().ToGuidString();
            Method = "EXEC";
            Timestamp = Time.Now;
            Data = new ConcurrentDictionary<string, string>();
            Client = new ClientContext();
            User = new UserContext();
        }

        public RequestContext(string uri) : this()
        {
            Uri = uri;
        }

        public RequestContext(Uri uri) : this()
        {
            Uri = uri.AbsoluteUri;
        }

        public RequestContext(string method, string uri) : this()
        {
            Method = method;
            Uri = uri;
        }

        public RequestContext(string method, Uri uri) : this()
        {
            Method = method;
            Uri = uri.AbsoluteUri;
        }

        public RequestContext(string method, string uri, DateTimeOffset startTime) : this()
        {
            Method = method;
            Uri = uri;
            Timestamp = startTime;
        }

        public RequestContext(string method, Uri uri, DateTimeOffset startTime) : this()
        {
            Method = method;
            Uri = uri.AbsoluteUri;
            Timestamp = startTime;
        }

        public RequestContext(string method, string uri, DateTimeOffset startTime, DateTimeOffset finishTime, bool? success)
            : this()
        {
            Method = method;
            Uri = uri;
            Timestamp = startTime;
            Duration = finishTime - startTime;
            Success = success;
        }

        public RequestContext(string method, Uri uri, DateTimeOffset startTime, DateTimeOffset finishTime, bool? success)
            : this()
        {
            Method = method;
            Uri = uri.AbsoluteUri;
            Timestamp = startTime;
            Duration = finishTime - startTime;
            Success = success;
        }

        public RequestContext(ServiceCallContext serviceCallContext)
            : this()
        {
            ParentId = serviceCallContext.SourceOperationId;
            Method = serviceCallContext.Method;
            Uri = serviceCallContext.Uri;

            Client.ClientId = serviceCallContext.ClientId;
            Client.ClientIp = serviceCallContext.ClientIp;
            Client.ClientLocation = serviceCallContext.ClientLocation;
            Client.ClientType = serviceCallContext.ClientType;
            Client.ClientModel = serviceCallContext.ClientModel;
            Client.ClientVersion = serviceCallContext.ClientVersion;

            User.UserId = serviceCallContext.AuthenticatedUserId.IsNotNullOrEmpty()
                ? serviceCallContext.AuthenticatedUserId
                : serviceCallContext.AnonymousUserId;
            User.Authenticated = serviceCallContext.AuthenticatedUserId.IsNotNullOrEmpty();
            User.UserName = serviceCallContext.UserName;
            User.SessionId = serviceCallContext.SessionId;

            if (serviceCallContext.Data.IsNotNullOrEmpty())
            {
                serviceCallContext.Data.ForEach(kv => Data[kv.Key] = kv.Value);
            }
        }

        public string Id { get; set; }

        public string ParentId { get; set; }

        /// <summary>
        ///     Request 的可阅读名称，一般为 $"{Method.ToUpperInvariant()} {Uri}".
        /// </summary>
        public string Name => $"{Method.ToUpperInvariant()} {Uri}";

        /// <summary>
        ///     Request 操作符。
        /// </summary>
        /// <example>
        ///     GET,POST,PUT,DELETE,EXEC
        /// </example>
        public string Method { get; set; }

        /// <summary>
        ///     Request 请求的资源标识符。
        /// </summary>
        public string Uri { get; set; }


        /// <summary>
        ///     Request 的创建使用，一般也被认为是请求的开始时间。
        /// </summary>
        public DateTimeOffset Timestamp { get; set; }

        /// <summary>
        ///     Request 从创建到被设置结果之间的时间。
        /// </summary>
        public TimeSpan? Duration { get; set; }

        public bool? Success { get; set; }

        public int ResponseCode
        {
            get => _responseCode.GetValueOrDefault(Success.HasValue ? Success.Value ? SuccessCode : FailureCode : 0);
            set => _responseCode = value;
        }


        public IDictionary<string, string> Data { get; set; }

        public ClientContext Client { get; }

        public UserContext User { get; }


        [JsonIgnore]
        public IServiceProvider Services
        {
            get => _serviceProvider;
            set
            {
                IRequestContextAccessor requestContextAccessor = value?.GetService<IRequestContextAccessor>();
                if (requestContextAccessor != null)
                {
                    requestContextAccessor.Request = this;
                }

                _serviceProvider = value;
            }
        }

        public void Start()
        {
            Timestamp = Time.Now;
        }

        public void Finish()
        {
            Duration = Time.Now - Timestamp;
        }

        public void Succeeded(int code = SuccessCode)
        {
            Success = true;
            ResponseCode = code;
            Finish();
        }

        public void Failed(int code = FailureCode)
        {
            Success = false;
            ResponseCode = code;
            Finish();
        }

        public ServiceCallContext GetServiceCallContext()
        {
            return new ServiceCallContext
            {
                SourceOperationId = ParentId,
                RequestName = Name,
                Method = Method,
                Uri = Uri,
                ClientId = Client.ClientId,
                ClientIp = Client.ClientIp,
                ClientLocation = Client.ClientLocation,
                ClientType = Client.ClientType,
                ClientModel = Client.ClientModel,
                ClientVersion = Client.ClientVersion,
                AuthenticatedUserId = User.Authenticated ? User.UserId : string.Empty,
                AnonymousUserId = User.UserId,
                UserName = User.UserName,
                SessionId = User.SessionId,
                Data = new Dictionary<string, string>(Data)
            };
        }

        public ServiceCallContext BuildChildServiceCallContext(string method, Uri uri)
        {
            return BuildChildServiceCallContext(method, uri.ToString());
        }

        public ServiceCallContext BuildChildServiceCallContext(string method, string uri)
        {
            return new ServiceCallContext
            {
                SourceOperationId = Id,
                RequestName = $"{method.ToUpperInvariant()} {uri}",
                Method = method,
                Uri = uri,
                ClientIp = Client.ClientIp,
                ClientLocation = Client.ClientLocation,
                ClientId = Client.ClientId,
                ClientType = Client.ClientType,
                ClientModel = Client.ClientModel,
                ClientVersion = Client.ClientVersion,
                AuthenticatedUserId = User.Authenticated ? User.UserId : string.Empty,
                AnonymousUserId = User.UserId,
                UserName = User.UserName,
                SessionId = User.SessionId,
                Data = new Dictionary<string, string>(Data)
            };
        } 
    }
}