﻿// ***********************************************************************
// Solution         : KC.Foundation
// Project          : KC.Foundation
// File             : Result.cs
// Updated          : 2017-06-01 09:11
// ***********************************************************************
// <copyright>
//     Copyright © 2016 - 2018 Kolibre Credit Team. All rights reserved.
// </copyright>
// ***********************************************************************

using System.Collections.Generic;
using System.Linq;

namespace KC.Foundation
{
    /// <summary>
    ///     Represents the result of an operation.
    /// </summary>
    public class Result
    {
        protected int? _code;
        protected string _message;

        /// <summary>
        ///     Flag indicating whether if the operation succeeded or not.
        /// </summary>
        /// <value>True if the operation succeeded, otherwise false.</value>
        public bool Succeeded { get; set; }

        /// <summary>
        ///     Gets or sets the code for this operation.
        /// </summary>
        public int Code
        {
            get => _code ?? (Succeeded ? 200 : Error?.Code ?? 500);
            set => _code = value;
        }

        /// <summary>
        ///     Gets or sets the message for this operation.
        /// </summary>
        public string Message
        {
            get => _message ?? (Succeeded ? "ok" : Error?.Message ?? "error");
            set => _message = value;
        }

        /// <summary>
        ///     The first error that occurred during the operation.
        /// </summary>
        /// <value>
        ///     If the <see cref="Succeeded" /> of this <see cref="Result" /> is <c>true</c>, return <c>null</c>.
        /// </value>
        public Error Error => Errors.FirstOrDefault();

        /// <summary>
        ///     An <see cref="List{T}" /> of <see cref="Error" />s containing an errors
        ///     that occurred during the operation.
        /// </summary>
        /// <value>An <see cref="List{T}" /> of <see cref="Error" />s.</value>
        public List<Error> Errors { get; } = new List<Error>();

        /// <summary>
        ///     Returns an <see cref="Result" /> indicating a successful operation.
        /// </summary>
        /// <returns>An <see cref="Result" /> indicating a successful operation.</returns>
        public static Result Success { get; } = new Result { Succeeded = true };

        /// <summary>
        ///     Creates a <see cref="Result" /> indicating a unsuccessfully operation, with a list of <paramref name="errors" /> if applicable.
        /// </summary>
        /// <param name="errors">An optional array of <see cref="Error" />s which caused the operation to fail.</param>
        /// <returns>An <see cref="Result" /> indicating a unsuccessfully operation, with a list of <paramref name="errors" /> if applicable.</returns>
        public static Result Failed(params Error[] errors)
        {
            Result result = new Result { Succeeded = false };
            if (errors != null)
            {
                result.Errors.AddRange(errors);
            }
            return result;
        }

        /// <summary>
        ///     Creates a <see cref="Result" /> indicating a unsuccessfully operation.
        /// </summary>
        /// <param name="code">A code of the <see cref="Error" /> which caused the operation to fail.</param>
        /// <param name="message">A message of the <see cref="Error" /> which caused the operation to fail.</param>
        /// <returns>An <see cref="Result" /> indicating a unsuccessfully operation.</returns>
        public static Result Failed(int code, string message)
        {
            Result result = new Result { Succeeded = false };
            message = message ?? "error";
            result.Errors.Add(new Error { Code = code, Message = message });
            return result;
        }

        /// <summary>
        ///     Creates a <see cref="Result" /> indicating a unsuccessfully operation.
        /// </summary>
        /// <param name="message">A message of the <see cref="Error" /> which caused the operation to fail.</param>
        /// <param name="code">A code of the <see cref="Error" /> which caused the operation to fail.</param>
        /// <returns>An <see cref="Result" /> indicating a unsuccessfully operation.</returns>
        public static Result Failed(string message, int code)
        {
            Result result = new Result { Succeeded = false };
            message = message ?? "error";
            result.Errors.Add(new Error { Code = code, Message = message });
            return result;
        }

        /// <summary>
        ///     Creates a <see cref="Result" /> indicating a succeeded operation.
        /// </summary>
        /// <param name="code">A code of the <see cref="Result" />.</param>
        /// <param name="message">A message of the <see cref="Result" />.</param>
        /// <returns>An <see cref="Result" /> indicating a succeeded operation.</returns>
        public static Result Succeed(int code, string message)
        {
            message = message ?? "ok";
            return new Result { Succeeded = true, Code = code, Message = message };
        }
    }

    /// <summary>
    ///     Represents the result of an operation.
    /// </summary>
    public class Result<TData> : Result where TData : class
    {
        /// <summary>
        ///     Gets or sets the data for this operation.
        /// </summary>
        public TData Data { get; set; }

        /// <summary>
        ///     Creates a <see cref="Result{TData}" /> indicating a succeeded operation.
        /// </summary>
        /// <param name="code">A code of the <see cref="Result" />.</param>
        /// <param name="message">A message of the <see cref="Result" />.</param>
        /// <param name="data">A <typeparamref name="TData" /> object contains the data.</param>
        /// <returns>An <see cref="Result" /> indicating a succeeded operation.</returns>
        public static Result<TData> Succeed(int code, string message, TData data)
        {
            message = message ?? "ok";
            return new Result<TData> { Succeeded = true, Code = code, Message = message, Data = data };
        }

        /// <summary>
        ///     Creates a <see cref="Result{TData}" /> indicating a succeeded operation.
        /// </summary>
        /// <param name="data">A <typeparamref name="TData" /> object contains the data.</param>
        /// <param name="message">A message of the <see cref="Result" />.</param>
        /// <param name="code">A code of the <see cref="Result" />.</param>
        /// <returns>An <see cref="Result" /> indicating a succeeded operation.</returns>
        public static Result<TData> Succeed(TData data, string message, int code)
        {
            message = message ?? "ok";
            return new Result<TData> { Succeeded = true, Code = code, Message = message, Data = data };
        }

        /// <summary>
        ///     Creates a <see cref="Result{TData}" /> indicating a succeeded operation.
        /// </summary>
        /// <param name="data">A <typeparamref name="TData" /> object contains the data.</param>
        /// <returns>An <see cref="Result" /> indicating a succeeded operation.</returns>
        public static Result<TData> Succeed(TData data)
        {
            return new Result<TData> { Succeeded = true, Code = 0, Message = "ok", Data = data };
        }
    }
}