﻿// ***********************************************************************
// Solution         : KC.Foundation
// Project          : KC.Foundation
// File             : ExceptionExtensions.cs
// Modified         : 2017-04-17  3:14 PM
// ***********************************************************************
// <copyright>
//     Copyright © 2016 - 2018 Kolibre Credit Team. All rights reserved.
// </copyright>
// ***********************************************************************

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Reflection;
using System.Text;

namespace KC.Foundation.Sys
{
    /// <summary>
    ///     <see cref="System.Exception" /> 的扩展类。
    /// </summary>
    public static class ExceptionExtensions
    {
        /// <summary>
        ///     将 <see cref="System.Reflection.ReflectionTypeLoadException" /> 转换为 <see cref="System.AggregateException" />
        /// </summary>
        public static AggregateException FlattenToAggregateException(this ReflectionTypeLoadException exception)
        {
            // if ReflectionTypeLoadException is thrown, we need to provide the
            // LoaderExceptions property in order to make it meaningful.
            List<Exception> all = new List<Exception> { exception };
            all.AddRange(exception.LoaderExceptions);
            throw new AggregateException("A ReflectionTypeLoadException has been thrown. The original exception and the contents of the LoaderExceptions property have been aggregated for your convenience.", all);
        }

        /// <summary>
        ///     获取 <see cref="System.Exception" /> 的详细错误信息。
        /// </summary>
        public static string GetExceptionString(this Exception exception)
        {
            StringBuilder sb = new StringBuilder();
            CreateExceptionString(sb, exception);
            return sb.ToString();
        }

        private static void CreateExceptionString(StringBuilder sb, Exception exception)
        {
            sb = sb ?? new StringBuilder();

            while (true)
            {
                if (exception == null)
                {
                    throw new ArgumentNullException(nameof(exception));
                }

                if (sb.Length > 0)
                {
                    sb.AppendLine("Inner Exception(s) Found:");
                }

                sb.AppendLine($"Type: {exception.GetType().FullName}");
                sb.AppendLine($"Message: {exception.Message}");
                sb.AppendLine($"DataJson: {exception.Data.ToJson("\\N", JsonSettings.LogJsonSettings)}");
                sb.AppendLine("Stacktrace:");
                sb.AppendLine($"{exception.StackTrace}");

                if (exception is ReflectionTypeLoadException loadException)
                {
                    Exception[] loaderExceptions = loadException.LoaderExceptions;
                    if (loaderExceptions.Length == 0)
                    {
                        sb.AppendLine("No LoaderExceptions found.");
                    }
                    else
                    {
                        foreach (Exception e in loaderExceptions)
                        {
                            CreateExceptionString(sb, e);
                        }
                    }
                }
                else if (exception is AggregateException)
                {
                    ReadOnlyCollection<Exception> innerExceptions = ((AggregateException)exception).InnerExceptions;
                    if (innerExceptions.Count == 0)
                    {
                        sb.AppendLine("No InnerExceptions found.");
                    }
                    else
                    {
                        foreach (Exception e in innerExceptions)
                        {
                            CreateExceptionString(sb, e);
                        }
                    }
                }
                else if (exception.InnerException != null)
                {
                    sb.AppendLine();
                    exception = exception.InnerException;
                    continue;
                }
                break;
            }
        }
    }
}